package com.diagbot.facade;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.RegularValueDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.entity.SysRegularConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.RegularConfigEnum;
import com.diagbot.model.entity.Allergy;
import com.diagbot.model.entity.AllergyMedicine;
import com.diagbot.model.label.PastLabel;
import com.diagbot.service.impl.SysRegularConfigServiceImpl;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.RegularConfigDataVO;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-11-30 16:10
 */
@Component
public class SysRegularConfigFacade extends SysRegularConfigServiceImpl {

    public List<RegularValueDTO> getRegularConfigDatas(RegularConfigDataVO regularConfigDataVO) {
        List<RegularValueDTO> listData = new ArrayList<>();
        QueryWrapper<SysRegularConfig> regularQuery = new QueryWrapper<>();
        regularQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq(regularConfigDataVO.getRulesCode() != null, "rules_code", regularConfigDataVO.getRulesCode())
                .eq(regularConfigDataVO.getRulesTepy() != null, "rules_tepy", regularConfigDataVO.getRulesTepy());
        List<SysRegularConfig> regularConfigs = list(regularQuery);
        if (ListUtil.isNotEmpty(regularConfigs)) {
            for (SysRegularConfig data : regularConfigs) {
                String pattern = "";
                String patternMax = "";
                if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                    pattern =  data.getRulesValue().split("￥")[0];
                    patternMax = data.getRulesValue().split("￥")[1];
                } else {
                    pattern = data.getRulesValue();
                }
                // 创建 Pattern 对象
                Pattern r = Pattern.compile(pattern, Pattern.CASE_INSENSITIVE);
                // 现在创建 matcher 对象
                Matcher m = r.matcher(regularConfigDataVO.getText());
                int trueMin = 0;
                while (m.find()) {
                    //System.out.println(m.group());
                    String steing = m.group();
                    String value = "";
                    Integer type = data.getRulesTepy();
                    if (data.getRulesTepy().equals(RegularConfigEnum.lis.getKey())) {
                        value = steing.replaceAll("([^\\d\\.]|\\.(?=[^\\.]*\\.))", "");
                        RegularValueDTO regularValueDTO = new RegularValueDTO();
                        regularValueDTO.setKey(data.getRulesName());
                        regularValueDTO.setType(type);
                        regularValueDTO.setValue(value);
                        listData.add(regularValueDTO);
                    } else if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                        //取否认的数
                        trueMin++;
                    }
                }
                int trueMax = 0;
                //取所有符合的
                if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                    Pattern rMax = Pattern.compile(patternMax, Pattern.CASE_INSENSITIVE);
                    // 现在创建 matcher 对象
                    Matcher mMax = rMax.matcher(regularConfigDataVO.getText());
                    while (mMax.find()) {
                        if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                            trueMax++;
                        }
                    }
                    int mun = trueMax - trueMin;
                    if (mun > 0) {
                        RegularValueDTO regularValueDTO = new RegularValueDTO();
                        regularValueDTO.setKey(data.getRulesName());
                        regularValueDTO.setType(data.getRulesTepy()
                        );
                        listData.add(regularValueDTO);
                    }
                }


            }


        }
        return listData;
    }

    /**
     * @param wordCrfDTO
     * @return
     */
    public WordCrfDTO getRegularConfigs(WordCrfDTO wordCrfDTO) {
        String symptom = wordCrfDTO.getSymptom();//现病史
        RegularConfigDataVO regularConfigData = new RegularConfigDataVO();
        regularConfigData.setText(symptom);
        List<RegularValueDTO> regularConfigDatas = getRegularConfigDatas(regularConfigData);
        List<Lis> lisData = wordCrfDTO.getLis();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();//获取过敏史数据
        List<AllergyMedicine> allergies = pastLabel.getAllergyMedicines();
        if (ListUtil.isNotEmpty(regularConfigDatas)) {
            for (RegularValueDTO data : regularConfigDatas) {
                if (data.getType().equals(RegularConfigEnum.lis.getKey())) {
                    Lis lis = new Lis();
                    lis.setUniqueName(data.getKey());
                    lis.setDetailName(data.getKey());
                    lis.setName(data.getKey());
                    lis.setValue(Double.valueOf(data.getValue()));
                    lisData.add(lis);
                } else if (data.getType().equals(RegularConfigEnum.allergy.getKey())) {
                    AllergyMedicine allergy = new AllergyMedicine();
                    allergy.setName(data.getKey());
                    allergy.setStandName(data.getKey());
                    allergies.add(allergy);
                }
            }
        }
        String pasts = wordCrfDTO.getPasts();//既往史
        RegularConfigDataVO regularConfigPasts = new RegularConfigDataVO();
        regularConfigData.setText(pasts);
        regularConfigData.setRulesTepy(RegularConfigEnum.allergy.getKey());
        List<RegularValueDTO> regularConfigPastss = getRegularConfigDatas(regularConfigData);
        if (ListUtil.isNotEmpty(regularConfigPastss)) {
            for(RegularValueDTO dataPasts : regularConfigPastss){
                AllergyMedicine allergy = new AllergyMedicine();
                allergy.setName(dataPasts.getKey());
                allergy.setStandName(dataPasts.getKey());
                allergies.add(allergy);
            }
        }
        pastLabel.setAllergyMedicines(allergies);
        wordCrfDTO.setPastLabel(pastLabel);
        wordCrfDTO.setLis(lisData);
        return wordCrfDTO;
    }
}
